<?php

declare(strict_types=1);

namespace Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\Action;

use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\ApiInterface as SogenactifApi;
use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\Generator\TransactionInterface;
use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\SealEncoderInterface;
use GuzzleHttp\Client;
use Http\Client\Exception\RequestException;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Exception\UnsupportedApiException;
use Payum\Core\GatewayAwareInterface;
use Payum\Core\GatewayAwareTrait;
use Payum\Core\Reply\HttpPostRedirect;
use Payum\Core\Request\Capture as PayumRequestCapture;
use Payum\Core\Security\GenericTokenFactoryAwareInterface;
use Payum\Core\Security\GenericTokenFactoryAwareTrait;
use Psr\Http\Message\ResponseInterface;
use RuntimeException;
use Sylius\Component\Core\Model\PaymentInterface as SyliusPaymentInterface;
use Symfony\Component\Routing\RouterInterface;

final class Capture implements
    ActionInterface,
    ApiAwareInterface,
    GenericTokenFactoryAwareInterface,
    GatewayAwareInterface
{
    use GenericTokenFactoryAwareTrait;
    use GatewayAwareTrait;

    public const CURRENCY_CODE_EURO = '978';

    public const INTERFACE_VERSION = 'IR_WS_2.3';

    public const ORDER_CHANNEL = 'INTERNET';

    private SogenactifApi $api;

    public function __construct(
        private readonly Client $client,
        private readonly SealEncoderInterface $sealEncoder,
        private readonly TransactionInterface $transaction,
        private readonly string $fqdnUrlInitializeTransaction,
        private readonly bool $shouldGenerateTransactionReference,
        private readonly RouterInterface $router,
    ) {
    }

    /**
     * @param PayumRequestCapture|mixed $request
     */
    public function execute($request): void
    {
        RequestNotSupportedException::assertSupports($this, $request);

        /** @var SyliusPaymentInterface $payment */
        $payment = $request->getModel();

        $requestToken = $request->getToken();

        $notifyToken = $this->tokenFactory->createNotifyToken(
            $requestToken?->getGatewayName(),
            $requestToken?->getDetails(),
        );

        $order = $payment->getOrder();

        if (null === $order) {
            throw new RuntimeException(sprintf('Could not resolve order for payment %s', $payment->getId()));
        }

        $channel = $order->getChannel();

        if (null === $channel) {
            throw new RuntimeException(sprintf('Could not resolve channel for order %s', $order->getId()));
        }

        // Data needed for create seal
        $infos = [
            'amount' => (string) $order->getTotal(),
            'automaticResponseUrl' => $notifyToken->getTargetUrl(),
            'currencyCode' => $this::CURRENCY_CODE_EURO,
            'interfaceVersion' => $this::INTERFACE_VERSION,
            'merchantId' => $this->api->getMerchantId(),
            'normalReturnUrl' => $requestToken?->getAfterUrl(),
            'orderChannel' => $this::ORDER_CHANNEL,
            'orderId' => $order->getId(),
        ];

        if ($this->shouldGenerateTransactionReference) {
            $infos['transactionReference'] = $this->transaction->generate($channel->getCode());
        }

        $infos['seal'] = $this->sealEncoder->createSeal($infos, $this->api->getSecretKey());
        $infos['keyVersion'] = $this->api->getKeyVersion();

        try {
            /** @var ResponseInterface $response */
            $response = $this
                ->client
                ->post($this->fqdnUrlInitializeTransaction, [
                    'json' => $infos,
                ]);

            $response = json_decode($response->getBody()->getContents(), true, 512, \JSON_THROW_ON_ERROR);

            throw new HttpPostRedirect(
                $response['redirectionUrl'],
                [
                    'redirectionVersion' => $response['redirectionVersion'],
                    'redirectionData' => $response['redirectionData'],
                ],
            );
        } catch (RequestException $exception) {
            $errorMessage = $exception->getMessage();
            $payment->setDetails(['error_message' => $errorMessage]);
        }

        throw new HttpPostRedirect($this->router->generate('sylius_shop_order_show', [
            'tokenValue' => $order->getTokenValue(),
        ]));
    }

    public function supports($request): bool
    {
        return
            $request instanceof PayumRequestCapture &&
            $request->getModel() instanceof SyliusPaymentInterface;
    }

    public function setApi($api): void
    {
        if (!$api instanceof SogenactifApi) {
            throw new UnsupportedApiException('Not supported. Expected an instance of ' . SogenactifApi::class);
        }

        $this->api = $api;
    }
}
